//
//  CalendarLayout.swift
//  Carleton150

import UIKit

struct CalendarLayoutConstants {
    struct Cell {
        
        // The height of the non-featured cell.
        static let standardHeight: CGFloat = 100
        
        // The height of the first visible cell.
        static let featuredHeight: CGFloat = 280
    }
}

class CalendarLayout: UICollectionViewFlowLayout {
    
    /** 
        The amount the user needs to scroll before the featured cell changes.
     */
    let dragOffset: CGFloat = 180.0
    
    var cache = [UICollectionViewLayoutAttributes]()
    
    /** 
        Returns the item index of the currently featured cell.
     */
    var featuredItemIndex: Int {
        get {
            // Ensures the featureItemIndex is never less than 0.
            return max(0, Int(collectionView!.contentOffset.y / dragOffset))
        }
    }
    
    /**
        Returns a value between 0 and 1 that represents 
        how close the next cell is to becoming the featured cell.
     */
    var nextItemPercentageOffset: CGFloat {
        get {
            return (collectionView!.contentOffset.y / dragOffset) - CGFloat(featuredItemIndex)
        }
    }
    
    /**
        Returns the width of the collection view.
     */
    var width: CGFloat {
        get {
            return CGRectGetWidth(collectionView!.bounds)
        }
    }
    
    /** 
        Returns the height of the collection view.
     */
    var height: CGFloat {
        get {
            return CGRectGetHeight(collectionView!.bounds)
        }
    }
    
    /** 
        Returns the number of items in the collection view.
     */
    var numberOfItems: Int {
        get {
            return collectionView!.numberOfItemsInSection(0)
        }
    }
    
    /**
        Return the size of all the content in the collection view.
     */
    override func collectionViewContentSize() -> CGSize {
        let contentHeight = (CGFloat(numberOfItems) * dragOffset) + (height - dragOffset)
        return CGSize(width: width, height: contentHeight)
    }
   
    /**
        Snaps the cells to navigation bar on the completion of a scroll.
     */
    override func targetContentOffsetForProposedContentOffset(proposedContentOffset: CGPoint, withScrollingVelocity velocity: CGPoint) -> CGPoint {
        let itemIndex = round(proposedContentOffset.y / dragOffset)
        let yOffset = itemIndex * dragOffset
        return CGPoint(x: 0, y: yOffset)
    }
    
    /**
        Sets the cell sizes for each cell as the collection of cells is moved, 
        adjusting the height of each cell as necessary.
     */
    override func prepareLayout() {
        cache.removeAll(keepCapacity: false)
        let standardHeight = CalendarLayoutConstants.Cell.standardHeight
        let featuredHeight = CalendarLayoutConstants.Cell.featuredHeight
        
        var frame = CGRectZero
        var y: CGFloat = 0
        
        for item in 0 ..< numberOfItems {
            let indexPath = NSIndexPath(forItem: item, inSection: 0)
            let attributes = UICollectionViewLayoutAttributes(forCellWithIndexPath: indexPath)
            
            attributes.zIndex = item
            var height = standardHeight
            
            if indexPath.item == featuredItemIndex {
                let yOffset = standardHeight * nextItemPercentageOffset
                y = collectionView!.contentOffset.y - yOffset
                height = featuredHeight
            } else if indexPath.item == (featuredItemIndex + 1) && indexPath.item != numberOfItems {
                let maxY = y + standardHeight
                height = standardHeight + max((featuredHeight - standardHeight) * nextItemPercentageOffset, 0)
                y = maxY - height
            }
            
            frame = CGRect(x: 0, y: y, width: width, height: height)
            attributes.frame = frame
            cache.append(attributes)
            y = CGRectGetMaxY(frame)
        }
    }
    
    /** 
        Return all attributes in the cache whose frame intersects with the rectangle passed to the method.
     */
    override func layoutAttributesForElementsInRect(rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        var layoutAttributes = [UICollectionViewLayoutAttributes]()
        for attributes in cache {
            if CGRectIntersectsRect(attributes.frame, rect) {
                layoutAttributes.append(attributes)
            }
        }
        return layoutAttributes
    }
    
    /** 
        Return true so that the layout is continuously invalidated (and thus updated)
        as the user scrolls.
     */
    override func shouldInvalidateLayoutForBoundsChange(newBounds: CGRect) -> Bool {
        return true
    }
    
}